--author: igor29381
--v5.2
--date: 19.11.18

ShowCargoUnits = {};

function ShowCargoUnits.prerequisitesPresent(specializations)
    return SpecializationUtil.hasSpecialization(Fillable, specializations);
end;

function ShowCargoUnits:preLoad(savegame)
	self.getSatelliteMap = ShowCargoUnits.getSatelliteMap;
	self.loadDefaultFillTypes = ShowCargoUnits.loadDefaultFillTypes;
	self.addFillTypesToCategory = ShowCargoUnits.addFillTypesToCategory;
	if hasXMLProperty(self.xmlFile, "vehicle.cargoUnits") then
		local categoryName = Utils.getNoNil(getXMLString(self.xmlFile, "vehicle.cargoUnits#categoryName"), string.format("%sCategory", self.typeName));
		local category = FillUtil.registerFillTypeCategory(categoryName);
		self.mapName, self.palletDesc = self:getSatelliteMap();
		if self.mapName then
			local key = string.format("vehicle.cargoUnits.%s", self.mapName);
			if hasXMLProperty(self.xmlFile, key) then
				self:addFillTypesToCategory(key, category);
			else
				self:loadDefaultFillTypes(category);
			end;
		else
			self:loadDefaultFillTypes(category);
		end;
	end;
end;

function ShowCargoUnits:loadDefaultFillTypes(category)
	local key = "vehicle.cargoUnits.default";
	if hasXMLProperty(self.xmlFile, key) then
		self:addFillTypesToCategory(key, category);
	else
		local defaultFillTypes = {"wool", "seeds", "fertilizer", "liquidFertilizer"};
		for i=1, #defaultFillTypes do
			local fillType = FillUtil.fillTypeNameToInt[defaultFillTypes[i]];
			if fillType then
				FillUtil.addFillTypeToCategory(category, fillType);
			end;
		end;
	end;
end;

function ShowCargoUnits:addFillTypesToCategory(key, category)
	local fillTypes = getXMLString(self.xmlFile, key.."#fillTypes");
	if fillTypes then
		fillTypes = Utils.splitString(" ", fillTypes);
		for i=1, #fillTypes do
			local fillType = FillUtil.fillTypeNameToInt[fillTypes[i]];
			if fillType then
				FillUtil.addFillTypeToCategory(category, fillType);
			end;
		end;
	end;
end;

function ShowCargoUnits:load(savegame)
	self.getMapNameIsValid = ShowCargoUnits.getMapNameIsValid;
	self.getHasMeasurement = ShowCargoUnits.getHasMeasurement;
	self.getValidPallet = ShowCargoUnits.getValidPallet;
	self.getTipOnPalletIsPossible = ShowCargoUnits.getTipOnPalletIsPossible;
	self.getLoadPalletIsPossible = ShowCargoUnits.getLoadPalletIsPossible;
	self.tipOnPallet = ShowCargoUnits.tipOnPallet;
	self.setUnitFillLevel = Utils.overwrittenFunction(self.setUnitFillLevel, ShowCargoUnits.setUnitFillLevel);
	if SpecializationUtil.hasSpecialization(Cover, self.specializations) then
		self.setCoverState = Utils.overwrittenFunction(self.setCoverState, ShowCargoUnits.setCoverState);
	end;
	self.getCanTipToGround = ShowCargoUnits.getCanTipToGround;
	if hasXMLProperty(self.xmlFile, "vehicle.cargoUnits") then
		self.cargoUnits = {};
		local i = 0;
		while true do
			local key = string.format("vehicle.cargoUnits.unit(%d)", i);
			local index = getXMLString(self.xmlFile, key.."#index");
			local fillTypes = getXMLString(self.xmlFile, key.."#fillTypes");
			local mapNames = getXMLString(self.xmlFile, key.."#mapNames");
			if not index or not fillTypes then
				break;
			end;
			if self:getMapNameIsValid(mapNames) then
				local node = Utils.indexToObject(self.components, index);
				if node and node > 0 then
					local count = getNumOfChildren(node);
					local unit = {node=node, count=count};
					fillTypes = Utils.splitString(" ", fillTypes);
					for ii=1, #fillTypes do
						local fillType = FillUtil.fillTypeNameToInt[fillTypes[ii]];
						if fillType then
							self.cargoUnits[fillType] = unit;
						end;
					end;
					if Utils.getNoNil(getXMLBool(self.xmlFile, key.."#defaultCargoUnit"), false) then
						self.defaultCargoUnit = unit;
					end;
				end;
			end;
			i = i + 1;
		end;
		if self.defaultCargoUnit then
			local fillTypes = self:getUnitFillTypes(1);
			for fillType,_ in pairs(fillTypes) do
				if not self.cargoUnits[fillType] then
					self.cargoUnits[fillType] = self.defaultCargoUnit;
				end;
			end;
		end;
		if self:getHasMeasurement() and hasXMLProperty(self.xmlFile, "vehicle.liquidColors") then
			local i = 0;
			self.liquidColors = {};
			while true do
				local key = string.format("vehicle.liquidColors.color(%d)", i);
				local rgb = getXMLString(self.xmlFile, key.."#rgb");
				local fillTypes = getXMLString(self.xmlFile, key.."#fillTypes");
				if not rgb or not fillTypes then
					break;
				end;
				fillTypes = Utils.splitString(" ", fillTypes);
				for ii=1, #fillTypes do
					local fillType = FillUtil.fillTypeNameToInt[fillTypes[ii]];
					if fillType then
						self.liquidColors[fillType] = {Utils.getVectorFromString(rgb)};
					end;
				end;
				i = i + 1;
			end;
		end;
		self.loadPalletMode = false;
		local palletReferencePointIndex = getXMLString(self.xmlFile, "vehicle.cargoUnits#palletReferencePointIndex");
		if palletReferencePointIndex then
			self.palletReferencePoint = Utils.indexToObject(self.components, palletReferencePointIndex);
		end;
		local tipOnPalletInputString = Utils.getNoNil(getXMLString(self.xmlFile, "vehicle.cargoUnits#tipOnPalletInputButton"), "TOGGLE_TIPSTATE_GROUND");
		self.tipOnPalletInputButton = InputBinding[tipOnPalletInputString];
		local loadPalletInputString = Utils.getNoNil(getXMLString(self.xmlFile, "vehicle.cargoUnits#loadPalletInputButton"), "IMPLEMENT_EXTRA4");
		self.loadPalletInputButton = InputBinding[loadPalletInputString];
		self.loadPalletText = {};
		self.loadPalletText[true] = g_i18n:getText("palletLoadModeOff");
		self.loadPalletText[false] = g_i18n:getText("palletLoadModeOn");
	end;
end;

function ShowCargoUnits:delete()
end;

function ShowCargoUnits:readStream(streamId, connection)
end;

function ShowCargoUnits:writeStream(streamId, connection)
end;

function ShowCargoUnits:mouseEvent(posX, posY, isDown, isUp, button)
end;

function ShowCargoUnits:keyEvent(unicode, sym, modifier, isDown)
end;

function ShowCargoUnits:update(dt)
	if self:getTipOnPalletIsPossible() then
		g_currentMission:addHelpButtonText(g_i18n:getText("tipOnPallet"), self.tipOnPalletInputButton);
		if InputBinding.hasEvent(self.tipOnPalletInputButton) then
			if g_server then
				self:tipOnPallet();
				self.loadPalletMode = false;
				g_server:broadcastEvent(CargoUnitsEvent:new(false, false, self));
			else
				g_client:getServerConnection():sendEvent(CargoUnitsEvent:new(false, true, self));
			end;
		end;
	end;
	if self:getLoadPalletIsPossible() then
		g_currentMission:addHelpButtonText(self.loadPalletText[self.loadPalletMode], self.loadPalletInputButton);
		if InputBinding.hasEvent(self.loadPalletInputButton) then
			self.loadPalletMode = not self.loadPalletMode;
			if g_server then
				g_server:broadcastEvent(CargoUnitsEvent:new(self.loadPalletMode, false, self));
			else
				g_client:getServerConnection():sendEvent(CargoUnitsEvent:new(self.loadPalletMode, false, self));
			end;
		end;
		if self.isServer then
			if self.loadPalletMode then
				local x,_,z = getWorldTranslation(self.palletReferencePoint);
				if g_currentMission.itemsToSave then
					for pallet,_ in pairs(g_currentMission.itemsToSave) do
						if pallet:isa(FillablePallet) and self:getValidPallet(pallet) then
							local fpx, _, fpz = getWorldTranslation(pallet.nodeId);
							local distance = Utils.vector2Length(fpx-x, fpz-z);
							if distance < 1.5 then
								local fillLevel = self:getFillLevel(pallet.fillType);
								local currentFillType = self.fillUnits[1].currentFillType;
								if pallet.fillType == FillUtil.FILLTYPE_SEEDS then
									if self.seedsExtension and pallet.seedsExtension then
										if currentFillType == FillUtil.FILLTYPE_UNKNOWN then
											self.seedsExtension.fruitType = pallet.seedsExtension.fruitType;
											self.seedsExtension.seedClass = pallet.seedsExtension.seedClass;
											g_server:broadcastEvent(GoldenSpike.ChangeSeedEvent:new(self.seedsExtension.fruitType, self.seedsExtension.seedClass, self));
										end;
									end;
								end;
								self:setFillLevel(fillLevel + pallet.fillLevel, pallet.fillType);
								pallet:delete();
								g_currentMission:addSharedMoney(-50, "wagePayment");
								g_currentMission:addMoneyChange(-50, FSBaseMission.MONEY_TYPE_SINGLE, true, g_i18n:getText("finance_wagePayment"));
							end;
						end;
					end;
				end;
			end;
		end;
	elseif self.loadPalletMode then
		if self.isServer then
			self.loadPalletMode = false;
			g_server:broadcastEvent(CargoUnitsEvent:new(false, false, self));
		end;
	end;
end;

function ShowCargoUnits:updateTick(dt)
end;

function ShowCargoUnits:draw()
end;

function ShowCargoUnits:getSatelliteMap()
	if Kavkaz and Kavkaz.PalletBase then
		return "Kavkaz", Kavkaz.PalletBase;
	end;
	if GoldenSpike and GoldenSpike.PalletBase then
		return "GoldenSpike", GoldenSpike.PalletBase;
	end;
end;

function ShowCargoUnits:getMapNameIsValid(mapNames)
	if not mapNames or mapNames == "" then
		return true;
	end;
	if not self.mapName then
		return false;
	end;
	mapNames = Utils.splitString(" ", mapNames);
	for i=1, #mapNames do
		if self.mapName == mapNames[i] then
			return true;
		end;
	end;
	return false;
end;

function ShowCargoUnits:getHasMeasurement()
	for i=1, #self.fillUnits do
		if self.fillUnits[i].measurementNodes then
			return true;
		end;
	end;
	return false;
end;

function ShowCargoUnits:getValidPallet(pallet)
	if pallet.fillLevel > self:getFreeCapacity() then
		return false;
	end;
	if not pallet.fillType then
		return false;
	end;
	if getUserAttribute(pallet.nodeId, "seedling") then
		return false;
	end;
	if not self.cargoUnits[pallet.fillType] then
		return false;
	end;
	local fillType = self.fillUnits[1].currentFillType;
	if fillType ~= 0 and pallet.fillType ~= fillType then
		return false;
	end;
	if pallet.fillLevel > self:getFreeCapacity() then
		return false;
	end;
	if pallet.fillType == FillUtil.FILLTYPE_SEEDS then
		if self.seedsExtension and pallet.seedsExtension then
			if fillType ~= 0 and (self.seedsExtension.fruitType ~= pallet.seedsExtension.fruitType
			or self.seedsExtension.seedClass ~= pallet.seedsExtension.seedClass) then
				return false;
			end;
		end;
	end;
	return true;
end;

function ShowCargoUnits:getTipOnPalletIsPossible()
	if not self.cargoUnits then
		return false;
	end;
	if not self.palletReferencePoint then
		return false;
	end;
	if self == g_currentMission.trailerInTipRange then
		return false;
	end;
	if not self:getIsActiveForInput() then
		return false;
	end;
	if not self.palletDesc then
		return false;
	end;
	if not self.palletDesc[self.fillUnits[1].currentFillType] then
		return false;
	end;
	if self.fillUnits[1].fillLevel <= 0 then
		return false;
	end;
	if SpecializationUtil.hasSpecialization(Cover, self.specializations) and not self.isCoverOpen then
		return false;
	end;
	return true;
end;

function ShowCargoUnits:getLoadPalletIsPossible()
	if not self.cargoUnits then
		return false;
	end;
	if self == g_currentMission.trailerInTipRange then
		return false;
	end;
	if not self:getIsActiveForInput() then
		return false;
	end;
	if not self.palletDesc then
		return false;
	end;
	if self:getFreeCapacity() <= 0 then
		return false;
	end;
	if SpecializationUtil.hasSpecialization(Cover, self.specializations) and not self.isCoverOpen then
		return false;
	end;
	return true;
end;

function ShowCargoUnits:tipOnPallet()
	local x,y,z = getWorldTranslation(self.palletReferencePoint);
	local rx,ry,rz = getWorldRotation(self.palletReferencePoint);
	if g_currentMission.itemsToSave then
		for pallet,_ in pairs(g_currentMission.itemsToSave) do
			if pallet:isa(FillablePallet) then
				local fpx, _, fpz = getWorldTranslation(pallet.nodeId);
				local distance = Utils.vector2Length(fpx-x, fpz-z);
				if distance < 1.5 then
					g_currentMission:showBlinkingWarning(g_i18n:getText("otherPallet"), 2000);
					return;
				end;
			end;
		end;
	end;
	local currentFillType = self.fillUnits[1].currentFillType;
	local fillLevel = self:getFillLevel(currentFillType);
	local pallet = FillablePallet:new(self.isServer, self.isClient);
	pallet.fillablePalletValueScale = 1;
	pallet:load(self.palletDesc[currentFillType], x,y+0.5,z, rx,ry,rz);
	pallet:register();
	pallet.fillType = currentFillType;
	if currentFillType == FillUtil.FILLTYPE_SEEDS then
		if self.seedsExtension then
			pallet.seedsExtension = {fruitType = self.seedsExtension.fruitType, seedClass = self.seedsExtension.seedClass};
			FillablePallet.setFruitPicture(pallet);
		end;
	end;
	local delta = math.min(2000, fillLevel);
	pallet:setFillLevel(delta, true);
	pallet.sendI3dFilename = true;
	fillLevel = fillLevel - delta;
	self:setFillLevel(fillLevel, currentFillType);
	self:resetFillLevelIfNeeded(currentFillType);
	if fillLevel <= 0.99 then
		self:setFillLevel(0, 0);
		self:resetFillLevelIfNeeded(0);
		if self.seedsExtension then
			self.seedsExtension.fruitType = 1;
			self.seedsExtension.seedClass = 3;
		end;
	end;
	g_currentMission:addSharedMoney(-50, "wagePayment");
	g_currentMission:addMoneyChange(-50, FSBaseMission.MONEY_TYPE_SINGLE, true, g_i18n:getText("finance_wagePayment"));
end;

function ShowCargoUnits:setCoverState(superFunc, isOpen, noEventSend)
	superFunc(self, isOpen, noEventSend);
	if not isOpen then
		self.loadPalletMode = false;
	end;
end;

function ShowCargoUnits:getCanTipToGround()
	return false;
end;

function ShowCargoUnits:setUnitFillLevel(superFunc, fillUnitIndex, fillLevel, fillType, force, fillInfo)
	superFunc(self, fillUnitIndex, fillLevel, fillType, force, fillInfo);
	if self:getFreeCapacity() <= 0 then
		self.loadPalletMode = false;
	end;
	local fillUnit = self.fillUnits[fillUnitIndex];
	if self.cargoUnits then
		if fillType ~= FillUtil.FILLTYPE_UNKNOWN then
			local unit = self.cargoUnits[fillType];
			local capacity = self:getCapacity(fillType);
			if unit and capacity > 0 then
				local count = math.ceil(unit.count*(fillLevel/capacity));
				for i=1, unit.count do
					setVisibility(getChildAt(unit.node, i-1), i <= count);
				end;
			end;
		else
			for _,unit in pairs(self.cargoUnits) do
				for i=1, unit.count do
					setVisibility(getChildAt(unit.node, i-1), false);
				end;
			end;
		end;
	end;
	if fillUnit.currentFillPlane ~= nil then
        for _, node in ipairs(fillUnit.currentFillPlane.nodes) do
            setVisibility(node.node, false);
        end
        fillUnit.currentFillPlane = nil;
    end
    if fillUnit.fillPlanes ~= nil and fillUnit.defaultFillPlane ~= nil then
        local fillPlane;
        local t = 0;
        if fillType ~= FillUtil.FILLTYPE_UNKNOWN then
            fillPlane = fillUnit.fillPlanes[fillType];
            t = fillUnit.fillLevel/fillUnit.capacity;
        end;
        if fillPlane == nil then
            fillPlane = fillUnit.defaultFillPlane;
        end;
        for _, node in ipairs(fillPlane.nodes) do
            local x,y,z, rx,ry,rz, sx,sy,sz = node.animCurve:get(t);
            setTranslation(node.node, x, y, z);
            setRotation(node.node, rx, ry, rz);
            setScale(node.node, sx, sy, sz);
            setVisibility(node.node, fillUnit.fillLevel > 0 or node.alwaysVisible);
        end;
        fillUnit.currentFillPlane = fillPlane;
    end;
	if self.liquidColors then
		for i=1, #self.fillUnits do
			local fillUnit = self.fillUnits[i];
			if fillUnit.measurementNodes then
				for ii=1, #fillUnit.measurementNodes do
					local rgb = self.liquidColors[fillUnit.currentFillType];
					if rgb then
						setShaderParameter(fillUnit.measurementNodes[ii], "liquidColor", rgb[1], rgb[2], rgb[3], 0, false);
					end;
				end;
			end;
		end;
	end;
end;

CargoUnitsEvent = {};
CargoUnitsEvent_mt = Class(CargoUnitsEvent, Event);
InitEventClass(CargoUnitsEvent, "CargoUnitsEvent");

function CargoUnitsEvent:emptyNew()
	local self = Event:new(CargoUnitsEvent_mt);
    return self;
end;

function CargoUnitsEvent:new(loadPalletMode, tipOnPallet, vehicle)
	local self = CargoUnitsEvent:emptyNew();
	self.loadPalletMode = loadPalletMode;
	self.tipOnPallet = tipOnPallet;
	self.vehicle = vehicle;
	return self;
end;

function CargoUnitsEvent:readStream(streamId, connection)
	local loadPalletMode = streamReadBool(streamId);
	local tipOnPallet = streamReadBool(streamId);
    local vehicle = networkGetObject(streamReadInt32(streamId));
	if vehicle then
		vehicle.loadPalletMode = loadPalletMode;
		if g_server then
			if tipOnPallet then
				vehicle:tipOnPallet();
			end;
			g_server:broadcastEvent(CargoUnitsEvent:new(vehicle.loadPalletMode, false, vehicle));
		end;
	end;
end;

function CargoUnitsEvent:writeStream(streamId, connection)
	streamWriteBool(streamId, self.loadPalletMode);
	streamWriteBool(streamId, self.tipOnPallet);
	streamWriteInt32(streamId, networkGetObjectId(self.vehicle));
end;
